const MinifyPlugin = require('babel-minify-webpack-plugin');
const HtmlPlugin = require('html-webpack-plugin');
const merge = require('webpack-merge');
const path = require('path');
const webpack = require('webpack');

const buildDir = path.resolve(__dirname, '../public');

const base = {
  context: __dirname,
  entry: {
    main: './main.js',
  },
  module: {
    rules: [
      {
        test: /\.js$/,
        exclude: /node_modules/,
        use: ['babel-loader'],
      },
    ],
  },
  devtool: 'source-map',
  plugins: [
    new HtmlPlugin({
      template: 'index.html',
    }),
  ],
};

const prod = {
  mode: 'production',
  output: {
    filename: '[name].[chunkhash].js',
    path: buildDir,
  },
  plugins: [
    new webpack.EnvironmentPlugin(
      Object.assign({
        NODE_ENV: 'production',
      })
    ),
    new MinifyPlugin(),
  ],
};

const dev = {
  mode: 'development',
  output: {
    filename: '[name].js',
    path: buildDir,
  },
  devtool: 'source-map',
  plugins: [
    new webpack.EnvironmentPlugin(
      Object.assign({
        NODE_ENV: 'development',
      })
    ),
  ],
  devServer: {
    contentBase: buildDir,
  },
};

module.exports = env => {
  let config;
  switch (env) {
    case 'prod': {
      config = merge(base, prod);
      break;
    }
    default: {
      config = merge(base, dev);
    }
  }

  return config;
};
