import PropTypes from 'prop-types';
import React from 'react';
import createReactClass from 'create-react-class';
import data from '../data/world-110m.json';
import {easeCubicInOut} from 'd3-ease';
import {feature} from 'topojson-client';
import {geoCircle, geoOrthographic, geoPath} from 'd3-geo';
import {interpolate} from 'd3-interpolate';
const land = feature(data, data.objects.land);
const Globe = createReactClass({
  displayName: 'Globe',
  propTypes: {
    center: PropTypes.array.isRequired,
    diameter: PropTypes.number.isRequired,
    landColor: PropTypes.string.isRequired,
    markerColor: PropTypes.string.isRequired,
    markerRadius: PropTypes.number.isRequired,
    oceanColor: PropTypes.string.isRequired,
    onClick: PropTypes.func,
    rotationTime: PropTypes.number.isRequired,
  },
  getDefaultProps: function getDefaultProps() {
    return {
      center: [0, 0],
      diameter: 60,
      landColor: '#191919',
      markerColor: '#009da5',
      markerRadius: 6,
      oceanColor: '#f2f2f2',
      rotationTime: 1000,
    };
  },
  componentDidMount: function componentDidMount() {
    this.update();
  },
  componentDidUpdate: function componentDidUpdate(prevProps) {
    if (
      prevProps.diameter !== this.props.diameter ||
      prevProps.landColor !== this.props.landColor ||
      prevProps.markerColor !== this.props.markerColor ||
      prevProps.markerRadius !== this.props.markerRadius ||
      prevProps.oceanColor !== this.props.oceanColor
    ) {
      this.update();
    }

    if (prevProps.center !== this.props.center) {
      this.recenter();
    }
  },
  update: function update() {
    const diameter = this.props.diameter;
    const radius = diameter / 2;
    const projection = geoOrthographic()
      .scale(radius - 2)
      .translate([radius, radius])
      .rotate(
        this.props.center.map(function(value) {
          return -value;
        })
      )
      .clipAngle(90);
    this.renderPath = geoPath()
      .projection(projection)
      .context(this.canvas.getContext('2d'));
    this.projection = projection;
    this.renderGlobe();
  },
  recenter: function recenter() {
    const _this = this;

    const start = Date.now();
    const duration = this.props.rotationTime;
    const getRotation = interpolate(
      this.projection.rotate(),
      this.props.center.map(function(value) {
        return -value;
      })
    );

    const updateRotation = function updateRotation() {
      const progress = Math.min(1, (Date.now() - start) / duration);

      _this.projection.rotate(getRotation(easeCubicInOut(progress)));

      _this.renderGlobe();

      if (progress < 1) {
        _this.transition = requestAnimationFrame(updateRotation);
      }
    };

    if (this.transition) {
      cancelAnimationFrame(this.transition);
      delete this.transition;
    }

    this.transition = requestAnimationFrame(updateRotation);
  },
  renderGlobe: function renderGlobe() {
    if (!this.canvas) {
      if (this.transition) {
        cancelAnimationFrame(this.transition);
      }

      return;
    }

    const context = this.canvas.getContext('2d');
    context.clearRect(0, 0, this.canvas.width, this.canvas.height);
    context.fillStyle = this.props.oceanColor;
    context.beginPath();
    this.renderPath({
      type: 'Sphere',
    });
    context.fill();
    context.fillStyle = this.props.landColor;
    context.beginPath();
    this.renderPath(land);
    context.fill();
    const circle = geoCircle()
      .center(this.props.center)
      .radius(this.props.markerRadius)();
    context.fillStyle = this.props.markerColor;
    context.beginPath();
    this.renderPath(circle);
    context.fill();
  },
  render: function render() {
    const _this2 = this;

    const style = {
      cursor: this.props.onClick ? 'pointer' : 'auto',
    };
    return React.createElement(
      'div',
      {
        className: 'pl-globe',
        onClick: this.props.onClick,
        style: style,
      },
      React.createElement('canvas', {
        height: this.props.diameter,
        ref: function ref(canvas) {
          _this2.canvas = canvas;
        },
        width: this.props.diameter,
      })
    );
  },
});
export default Globe;
